package com.example.cameraxtestappjava;

import static com.example.cameraxtestappjava.segpass.SegpassCamera.REQUEST_CAMERA_PERMISSION;

import android.annotation.SuppressLint;
import android.content.pm.ActivityInfo;
import android.os.Bundle;

import androidx.activity.EdgeToEdge;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;

import android.Manifest;
import android.content.pm.PackageManager;


import com.example.cameraxtestappjava.segpass.camera.view.AutoFitTextureView;
import com.example.cameraxtestappjava.segpass.camera.utils.SegpassCameraCallback;
import com.example.cameraxtestappjava.segpass.camera.utils.SegpassPermissionCallback;
import com.example.cameraxtestappjava.databinding.ActivityCameraNewBinding;
import com.example.cameraxtestappjava.segpass.SegpassCamera;

public class CameraActivityNew extends AppCompatActivity implements ActivityCompat.OnRequestPermissionsResultCallback, SegpassCameraCallback, SegpassPermissionCallback {

    private static final String TAG = "CameraActivityNew";
    ActivityCameraNewBinding binding;
    SegpassCamera mSegpassCamera;
    AutoFitTextureView mTextureView;

    @SuppressLint("SourceLockedOrientationActivity")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        binding = ActivityCameraNewBinding.inflate(getLayoutInflater());
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        setContentView(binding.getRoot());

        mTextureView = binding.inCamera2.tvCameraTextureView;
        mSegpassCamera = new SegpassCamera(this, mTextureView, this, this);
        setUpListeners();

        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });
    }

    @Override
    public void onResume() {
        super.onResume();
        mSegpassCamera.resumeCamera();
    }

    @Override
    public void onPause() {
        mSegpassCamera.pauseCamera();
        super.onPause();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        if (requestCode == REQUEST_CAMERA_PERMISSION) {
            if (grantResults.length != 2 || grantResults[0] != PackageManager.PERMISSION_GRANTED) {
                onPermissionDenied();
            }
        } else {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    private void setUpListeners() {
        binding.inCamera2.btnTakepicture.setOnClickListener(v -> mSegpassCamera.takePicture());
    }

    /**
     * Camera callback
     */

    @Override
    public void onPictureTakenSuccess(String base64) {
        mSegpassCamera.showToast("Base64 generated.");
    }

    @Override
    public void onPictureTakenFailError(String error) {
        mSegpassCamera.showToast(error);
    }

    /**
     * Camera state callback (just to validate if there was an error while initializing the camera)
     */

    @Override
    public void onCameraInitError(String errorMessage) {
        mSegpassCamera.showToast(errorMessage);
    }

    /**
     * Permissions callbck
     */

    @Override
    public void onPermissionRequest() {
        requestPermissions(new String[]{Manifest.permission.CAMERA, Manifest.permission.WRITE_EXTERNAL_STORAGE}, REQUEST_CAMERA_PERMISSION);
    }

    @Override
    public void onPermissionGranted() {
        // Do nothing, use camera.
    }

    @Override
    public void onPermissionDenied() {
        mSegpassCamera.showToast("No permissions granted, closing camera.");
        finish();
    }
}